@@ -3,20 +3,20 @@
    Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2006, 2007
    Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* This file contains the functions to evaluate integer expressions for
@@ -62,19 +62,19 @@ typedef enum eval_error
   }
 eval_error;
 
-static eval_error logical_or_term (eval_token, int32_t *);
-static eval_error logical_and_term (eval_token, int32_t *);
-static eval_error or_term (eval_token, int32_t *);
-static eval_error xor_term (eval_token, int32_t *);
-static eval_error and_term (eval_token, int32_t *);
-static eval_error equality_term (eval_token, int32_t *);
-static eval_error cmp_term (eval_token, int32_t *);
-static eval_error shift_term (eval_token, int32_t *);
-static eval_error add_term (eval_token, int32_t *);
-static eval_error mult_term (eval_token, int32_t *);
-static eval_error exp_term (eval_token, int32_t *);
-static eval_error unary_term (eval_token, int32_t *);
-static eval_error simple_term (eval_token, int32_t *);
+static eval_error logical_or_term (const char *, eval_token, int32_t *);
+static eval_error logical_and_term (const char *, eval_token, int32_t *);
+static eval_error or_term (const char *, eval_token, int32_t *);
+static eval_error xor_term (const char *, eval_token, int32_t *);
+static eval_error and_term (const char *, eval_token, int32_t *);
+static eval_error equality_term (const char *, eval_token, int32_t *);
+static eval_error cmp_term (const char *, eval_token, int32_t *);
+static eval_error shift_term (const char *, eval_token, int32_t *);
+static eval_error add_term (const char *, eval_token, int32_t *);
+static eval_error mult_term (const char *, eval_token, int32_t *);
+static eval_error exp_term (const char *, eval_token, int32_t *);
+static eval_error unary_term (const char *, eval_token, int32_t *);
+static eval_error simple_term (const char *, eval_token, int32_t *);
 
 /*--------------------.
 | Lexical functions.  |
@@ -287,14 +287,14 @@ eval_lex (int32_t *val)
 `---------------------------------------*/
 
 bool
-evaluate (const char *expr, int32_t *val)
+evaluate (const char *me, const char *expr, int32_t *val)
 {
   eval_token et;
   eval_error err;
 
   eval_init_lex (expr);
   et = eval_lex (val);
-  err = logical_or_term (et, val);
+  err = logical_or_term (me, et, val);
 
   if (err == NO_ERROR && *eval_text != '\0')
     {
@@ -310,50 +310,40 @@ evaluate (const char *expr, int32_t *val)
       break;
 
     case MISSING_RIGHT:
-      M4ERROR ((warning_status, 0,
-		"bad expression in eval (missing right parenthesis): %s",
-		expr));
+      m4_warn (0, me, _("bad expression (missing right parenthesis): %s"),
+	       expr);
       break;
 
     case SYNTAX_ERROR:
-      M4ERROR ((warning_status, 0,
-		"bad expression in eval: %s", expr));
+      m4_warn (0, me, _("bad expression: %s"), expr);
       break;
 
     case UNKNOWN_INPUT:
-      M4ERROR ((warning_status, 0,
-		"bad expression in eval (bad input): %s", expr));
+      m4_warn (0, me, _("bad expression (bad input): %s"), expr);
       break;
 
     case EXCESS_INPUT:
-      M4ERROR ((warning_status, 0,
-		"bad expression in eval (excess input): %s", expr));
+      m4_warn (0, me, _("bad expression (excess input): %s"), expr);
       break;
 
     case INVALID_OPERATOR:
-      M4ERROR ((warning_status, 0,
-		"invalid operator in eval: %s", expr));
-      retcode = EXIT_FAILURE;
+      m4_error (0, 0, me, _("invalid operator: %s"), expr);
       break;
 
     case DIVIDE_ZERO:
-      M4ERROR ((warning_status, 0,
-		"divide by zero in eval: %s", expr));
+      m4_warn (0, me, _("divide by zero: %s"), expr);
       break;
 
     case MODULO_ZERO:
-      M4ERROR ((warning_status, 0,
-		"modulo by zero in eval: %s", expr));
+      m4_warn (0, me, _("modulo by zero: %s"), expr);
       break;
 
     case NEGATIVE_EXPONENT:
-      M4ERROR ((warning_status, 0,
-		"negative exponent in eval: %s", expr));
+      m4_warn (0, me, _("negative exponent: %s"), expr);
       break;
 
     default:
-      M4ERROR ((warning_status, 0,
-		"INTERNAL ERROR: bad error code in evaluate ()"));
+      assert (!"evaluate");
       abort ();
     }
 
@@ -365,12 +355,12 @@ evaluate (const char *expr, int32_t *val)
 `---------------------------*/
 
 static eval_error
-logical_or_term (eval_token et, int32_t *v1)
+logical_or_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
 
-  if ((er = logical_and_term (et, v1)) != NO_ERROR)
+  if ((er = logical_and_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == LOR)
@@ -380,7 +370,7 @@ logical_or_term (eval_token et, int32_t *v1)
 	return UNKNOWN_INPUT;
 
       /* Implement short-circuiting of valid syntax.  */
-      er = logical_and_term (et, &v2);
+      er = logical_and_term (me, et, &v2);
       if (er == NO_ERROR)
 	*v1 = *v1 || v2;
       else if (*v1 != 0 && er < SYNTAX_ERROR)
@@ -396,12 +386,12 @@ logical_or_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-logical_and_term (eval_token et, int32_t *v1)
+logical_and_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
 
-  if ((er = or_term (et, v1)) != NO_ERROR)
+  if ((er = or_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == LAND)
@@ -411,7 +401,7 @@ logical_and_term (eval_token et, int32_t *v1)
 	return UNKNOWN_INPUT;
 
       /* Implement short-circuiting of valid syntax.  */
-      er = or_term (et, &v2);
+      er = or_term (me, et, &v2);
       if (er == NO_ERROR)
 	*v1 = *v1 && v2;
       else if (*v1 == 0 && er < SYNTAX_ERROR)
@@ -427,12 +417,12 @@ logical_and_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-or_term (eval_token et, int32_t *v1)
+or_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
 
-  if ((er = xor_term (et, v1)) != NO_ERROR)
+  if ((er = xor_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == OR)
@@ -441,7 +431,7 @@ or_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = xor_term (et, &v2)) != NO_ERROR)
+      if ((er = xor_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       *v1 |= v2;
@@ -454,12 +444,12 @@ or_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-xor_term (eval_token et, int32_t *v1)
+xor_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
 
-  if ((er = and_term (et, v1)) != NO_ERROR)
+  if ((er = and_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == XOR)
@@ -468,7 +458,7 @@ xor_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = and_term (et, &v2)) != NO_ERROR)
+      if ((er = and_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       *v1 ^= v2;
@@ -481,12 +471,12 @@ xor_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-and_term (eval_token et, int32_t *v1)
+and_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
 
-  if ((er = equality_term (et, v1)) != NO_ERROR)
+  if ((er = equality_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == AND)
@@ -495,7 +485,7 @@ and_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = equality_term (et, &v2)) != NO_ERROR)
+      if ((er = equality_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       *v1 &= v2;
@@ -508,13 +498,13 @@ and_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-equality_term (eval_token et, int32_t *v1)
+equality_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token op;
   int32_t v2;
   eval_error er;
 
-  if ((er = cmp_term (et, v1)) != NO_ERROR)
+  if ((er = cmp_term (me, et, v1)) != NO_ERROR)
     return er;
 
   /* In the 1.4.x series, we maintain the traditional behavior that
@@ -526,13 +516,12 @@ equality_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = cmp_term (et, &v2)) != NO_ERROR)
+      if ((er = cmp_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       if (op == ASSIGN)
       {
-	M4ERROR ((warning_status, 0, "\
-Warning: recommend ==, not =, for equality operator"));
+	m4_warn (0, me, _("recommend ==, not =, for equality"));
 	op = EQ;
       }
       *v1 = (op == EQ) == (*v1 == v2);
@@ -545,13 +534,13 @@ Warning: recommend ==, not =, for equality operator"));
 }
 
 static eval_error
-cmp_term (eval_token et, int32_t *v1)
+cmp_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token op;
   int32_t v2;
   eval_error er;
 
-  if ((er = shift_term (et, v1)) != NO_ERROR)
+  if ((er = shift_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((op = eval_lex (&v2)) == GT || op == GTEQ
@@ -562,7 +551,7 @@ cmp_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = shift_term (et, &v2)) != NO_ERROR)
+      if ((er = shift_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       switch (op)
@@ -584,8 +573,7 @@ cmp_term (eval_token et, int32_t *v1)
 	  break;
 
 	default:
-	  M4ERROR ((warning_status, 0,
-		    "INTERNAL ERROR: bad comparison operator in cmp_term ()"));
+	  assert (!"cmp_term");
 	  abort ();
 	}
     }
@@ -597,14 +585,14 @@ cmp_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-shift_term (eval_token et, int32_t *v1)
+shift_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token op;
   int32_t v2;
   uint32_t u1;
   eval_error er;
 
-  if ((er = add_term (et, v1)) != NO_ERROR)
+  if ((er = add_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((op = eval_lex (&v2)) == LSHIFT || op == RSHIFT)
@@ -614,7 +602,7 @@ shift_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = add_term (et, &v2)) != NO_ERROR)
+      if ((er = add_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       /* Minimize undefined C behavior (shifting by a negative number,
@@ -638,8 +626,7 @@ shift_term (eval_token et, int32_t *v1)
 	  break;
 
 	default:
-	  M4ERROR ((warning_status, 0,
-		    "INTERNAL ERROR: bad shift operator in shift_term ()"));
+	  assert (!"shift_term");
 	  abort ();
 	}
     }
@@ -651,13 +638,13 @@ shift_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-add_term (eval_token et, int32_t *v1)
+add_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token op;
   int32_t v2;
   eval_error er;
 
-  if ((er = mult_term (et, v1)) != NO_ERROR)
+  if ((er = mult_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((op = eval_lex (&v2)) == PLUS || op == MINUS)
@@ -666,7 +653,7 @@ add_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = mult_term (et, &v2)) != NO_ERROR)
+      if ((er = mult_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       /* Minimize undefined C behavior on overflow.  This code assumes
@@ -686,13 +673,13 @@ add_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-mult_term (eval_token et, int32_t *v1)
+mult_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token op;
   int32_t v2;
   eval_error er;
 
-  if ((er = exp_term (et, v1)) != NO_ERROR)
+  if ((er = exp_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((op = eval_lex (&v2)) == TIMES || op == DIVIDE || op == MODULO)
@@ -701,7 +688,7 @@ mult_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = exp_term (et, &v2)) != NO_ERROR)
+      if ((er = exp_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       /* Minimize undefined C behavior on overflow.  This code assumes
@@ -735,8 +722,7 @@ mult_term (eval_token et, int32_t *v1)
 	  break;
 
 	default:
-	  M4ERROR ((warning_status, 0,
-		    "INTERNAL ERROR: bad operator in mult_term ()"));
+	  assert (!"mult_term");
 	  abort ();
 	}
     }
@@ -748,13 +734,13 @@ mult_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-exp_term (eval_token et, int32_t *v1)
+exp_term (const char *me, eval_token et, int32_t *v1)
 {
   uint32_t result;
   int32_t v2;
   eval_error er;
 
-  if ((er = unary_term (et, v1)) != NO_ERROR)
+  if ((er = unary_term (me, et, v1)) != NO_ERROR)
     return er;
 
   while ((et = eval_lex (&v2)) == EXPONENT)
@@ -763,7 +749,7 @@ exp_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = exp_term (et, &v2)) != NO_ERROR)
+      if ((er = exp_term (me, et, &v2)) != NO_ERROR)
 	return er;
 
       /* Minimize undefined C behavior on overflow.  This code assumes
@@ -787,7 +773,7 @@ exp_term (eval_token et, int32_t *v1)
 }
 
 static eval_error
-unary_term (eval_token et, int32_t *v1)
+unary_term (const char *me, eval_token et, int32_t *v1)
 {
   eval_token et2 = et;
   eval_error er;
@@ -798,7 +784,7 @@ unary_term (eval_token et, int32_t *v1)
       if (et2 == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = unary_term (et2, v1)) != NO_ERROR)
+      if ((er = unary_term (me, et2, v1)) != NO_ERROR)
 	return er;
 
       /* Minimize undefined C behavior on overflow.  This code assumes
@@ -812,14 +798,14 @@ unary_term (eval_token et, int32_t *v1)
       else if (et == LNOT)
 	*v1 = *v1 == 0 ? 1 : 0;
     }
-  else if ((er = simple_term (et, v1)) != NO_ERROR)
+  else if ((er = simple_term (me, et, v1)) != NO_ERROR)
     return er;
 
   return NO_ERROR;
 }
 
 static eval_error
-simple_term (eval_token et, int32_t *v1)
+simple_term (const char *me, eval_token et, int32_t *v1)
 {
   int32_t v2;
   eval_error er;
@@ -831,7 +817,7 @@ simple_term (eval_token et, int32_t *v1)
       if (et == ERROR)
 	return UNKNOWN_INPUT;
 
-      if ((er = logical_or_term (et, v1)) != NO_ERROR)
+      if ((er = logical_or_term (me, et, v1)) != NO_ERROR)
 	return er;
 
       et = eval_lex (&v2);
