@@ -1,22 +1,22 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 2004, 2006 Free Software
-   Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 2004, 2006, 2007 Free
+   Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Handling of path search of included files via the builtins "include"
@@ -101,7 +101,7 @@ add_include_directory (const char *dir)
   dir_list_end = incl;
 
 #ifdef DEBUG_INCL
-  fprintf (stderr, "add_include_directory (%s);\n", dir);
+  xfprintf (stderr, "add_include_directory (%s);\n", dir);
 #endif
 }
 
@@ -133,8 +133,7 @@ m4_path_search (const char *file, char **result)
   if (fp != NULL)
     {
       if (set_cloexec_flag (fileno (fp), true) != 0)
-	M4ERROR ((warning_status, errno,
-		  "Warning: cannot protect input file across forks"));
+	m4_warn (errno, NULL, _("cannot protect input file across forks"));
       if (result)
 	*result = xstrdup (file);
       return fp;
@@ -154,7 +153,7 @@ m4_path_search (const char *file, char **result)
       strcpy (name + incl->len + 1, file);
 
 #ifdef DEBUG_INCL
-      fprintf (stderr, "m4_path_search (%s) -- trying %s\n", file, name);
+      xfprintf (stderr, "m4_path_search (%s) -- trying %s\n", file, name);
 #endif
 
       fp = fopen (name, "r");
@@ -163,8 +162,7 @@ m4_path_search (const char *file, char **result)
 	  if (debug_level & DEBUG_TRACE_PATH)
 	    DEBUG_MESSAGE2 ("path search for `%s' found `%s'", file, name);
 	  if (set_cloexec_flag (fileno (fp), true) != 0)
-	    M4ERROR ((warning_status, errno,
-		      "Warning: cannot protect input file across forks"));
+	    m4_warn (errno, NULL, _("cannot protect input file across forks"));
 	  if (result)
 	    *result = name;
 	  else
@@ -185,9 +183,9 @@ include_dump (void)
 {
   includes *incl;
 
-  fprintf (stderr, "include_dump:\n");
+  xfprintf (stderr, "include_dump:\n");
   for (incl = dir_list; incl != NULL; incl = incl->next)
-    fprintf (stderr, "\t%s\n", incl->dir);
+    xfprintf (stderr, "\t%s\n", incl->dir);
 }
 
 #endif /* DEBUG_INCL */
