@@ -1,22 +1,22 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2003, 2006 Free
-   Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2003, 2006, 2007, 2008
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* This file handles all the low level work around the symbol table.  The
@@ -32,7 +32,6 @@
    will then always be the first found.  */
 
 #include "m4.h"
-#include <limits.h>
 
 #ifdef DEBUG_SYM
 /* When evaluating hash table performance, this profiling code shows
@@ -56,10 +55,10 @@ show_profile (void)
   int i;
   for (i = 0; i < 5; i++)
     {
-      fprintf(stderr, "m4: lookup mode %d called %d times, %d compares, "
-	      "%d misses, %lld bytes\n",
-	      i, profiles[i].entry, profiles[i].comparisons,
-	      profiles[i].misses, profiles[i].bytes);
+      xfprintf(stderr, "m4: lookup mode %d called %d times, %d compares, "
+	       "%d misses, %lld bytes\n",
+	       i, profiles[i].entry, profiles[i].comparisons,
+	       profiles[i].misses, profiles[i].bytes);
     }
 }
 
@@ -101,18 +100,13 @@ symtab_init (void)
   size_t i;
   symbol **s;
 
-  s = symtab = (symbol **) xmalloc (hash_table_size * sizeof (symbol *));
+  s = symtab = (symbol **) xnmalloc (hash_table_size, sizeof (symbol *));
 
   for (i = 0; i < hash_table_size; i++)
     s[i] = NULL;
 
 #ifdef DEBUG_SYM
-  {
-    int e = atexit(show_profile);
-    if (e != 0)
-      M4ERROR ((warning_status, 0,
-		"INTERNAL ERROR: unable to show symtab profile"));
-  }
+  atexit (show_profile); /* Ignore failure, since this is debug code.  */
 #endif /* DEBUG_SYM */
 }
 
@@ -141,14 +135,13 @@ void
 free_symbol (symbol *sym)
 {
   if (SYMBOL_PENDING_EXPANSIONS (sym) > 0)
-    SYMBOL_DELETED (sym) = TRUE;
+    SYMBOL_DELETED (sym) = true;
   else
     {
-      if (SYMBOL_NAME (sym))
-	xfree (SYMBOL_NAME (sym));
+      free (SYMBOL_NAME (sym));
       if (SYMBOL_TYPE (sym) == TOKEN_TEXT)
-	xfree (SYMBOL_TEXT (sym));
-      xfree (sym);
+	free (SYMBOL_TEXT (sym));
+      free (sym);
     }
 }
 
@@ -212,16 +205,16 @@ lookup_symbol (const char *name, symbol_lookup mode)
 	  if (SYMBOL_PENDING_EXPANSIONS (sym) > 0)
 	    {
 	      symbol *old = sym;
-	      SYMBOL_DELETED (old) = TRUE;
+	      SYMBOL_DELETED (old) = true;
 
 	      sym = (symbol *) xmalloc (sizeof (symbol));
 	      SYMBOL_TYPE (sym) = TOKEN_VOID;
 	      SYMBOL_TRACED (sym) = SYMBOL_TRACED (old);
 	      SYMBOL_NAME (sym) = xstrdup (name);
-	      SYMBOL_SHADOWED (sym) = FALSE;
-	      SYMBOL_MACRO_ARGS (sym) = FALSE;
-	      SYMBOL_BLIND_NO_ARGS (sym) = FALSE;
-	      SYMBOL_DELETED (sym) = FALSE;
+	      SYMBOL_SHADOWED (sym) = false;
+	      SYMBOL_MACRO_ARGS (sym) = false;
+	      SYMBOL_BLIND_NO_ARGS (sym) = false;
+	      SYMBOL_DELETED (sym) = false;
 	      SYMBOL_PENDING_EXPANSIONS (sym) = 0;
 
 	      SYMBOL_NEXT (sym) = SYMBOL_NEXT (old);
@@ -240,12 +233,12 @@ lookup_symbol (const char *name, symbol_lookup mode)
 
       sym = (symbol *) xmalloc (sizeof (symbol));
       SYMBOL_TYPE (sym) = TOKEN_VOID;
-      SYMBOL_TRACED (sym) = FALSE;
+      SYMBOL_TRACED (sym) = false;
       SYMBOL_NAME (sym) = xstrdup (name);
-      SYMBOL_SHADOWED (sym) = FALSE;
-      SYMBOL_MACRO_ARGS (sym) = FALSE;
-      SYMBOL_BLIND_NO_ARGS (sym) = FALSE;
-      SYMBOL_DELETED (sym) = FALSE;
+      SYMBOL_SHADOWED (sym) = false;
+      SYMBOL_MACRO_ARGS (sym) = false;
+      SYMBOL_BLIND_NO_ARGS (sym) = false;
+      SYMBOL_DELETED (sym) = false;
       SYMBOL_PENDING_EXPANSIONS (sym) = 0;
 
       SYMBOL_NEXT (sym) = *spp;
@@ -253,7 +246,7 @@ lookup_symbol (const char *name, symbol_lookup mode)
 
       if (mode == SYMBOL_PUSHDEF && cmp == 0)
 	{
-	  SYMBOL_SHADOWED (SYMBOL_NEXT (sym)) = TRUE;
+	  SYMBOL_SHADOWED (SYMBOL_NEXT (sym)) = true;
 	  SYMBOL_TRACED (sym) = SYMBOL_TRACED (SYMBOL_NEXT (sym));
 	}
       return sym;
@@ -271,12 +264,12 @@ lookup_symbol (const char *name, symbol_lookup mode)
       if (cmp != 0 || sym == NULL)
 	return NULL;
       {
-	boolean traced = FALSE;
+	bool traced = false;
 	if (SYMBOL_NEXT (sym) != NULL
 	    && SYMBOL_SHADOWED (SYMBOL_NEXT (sym))
 	    && mode == SYMBOL_POPDEF)
 	  {
-	    SYMBOL_SHADOWED (SYMBOL_NEXT (sym)) = FALSE;
+	    SYMBOL_SHADOWED (SYMBOL_NEXT (sym)) = false;
 	    SYMBOL_TRACED (SYMBOL_NEXT (sym)) = SYMBOL_TRACED (sym);
 	  }
 	else
@@ -293,12 +286,12 @@ lookup_symbol (const char *name, symbol_lookup mode)
 	  {
 	    sym = (symbol *) xmalloc (sizeof (symbol));
 	    SYMBOL_TYPE (sym) = TOKEN_VOID;
-	    SYMBOL_TRACED (sym) = TRUE;
+	    SYMBOL_TRACED (sym) = true;
 	    SYMBOL_NAME (sym) = xstrdup (name);
-	    SYMBOL_SHADOWED (sym) = FALSE;
-	    SYMBOL_MACRO_ARGS (sym) = FALSE;
-	    SYMBOL_BLIND_NO_ARGS (sym) = FALSE;
-	    SYMBOL_DELETED (sym) = FALSE;
+	    SYMBOL_SHADOWED (sym) = false;
+	    SYMBOL_MACRO_ARGS (sym) = false;
+	    SYMBOL_BLIND_NO_ARGS (sym) = false;
+	    SYMBOL_DELETED (sym) = false;
 	    SYMBOL_PENDING_EXPANSIONS (sym) = 0;
 
 	    SYMBOL_NEXT (sym) = *spp;
@@ -308,8 +301,7 @@ lookup_symbol (const char *name, symbol_lookup mode)
       return NULL;
 
     default:
-      M4ERROR ((warning_status, 0,
-		"INTERNAL ERROR: invalid mode to symbol_lookup ()"));
+      assert (!"symbol_lookup");
       abort ();
     }
 }
@@ -326,7 +318,7 @@ lookup_symbol (const char *name, symbol_lookup mode)
 `-----------------------------------------------------------------*/
 
 void
-hack_all_symbols (hack_symbol *func, const char *data)
+hack_all_symbols (hack_symbol *func, void *data)
 {
   size_t h;
   symbol *sym;
@@ -340,7 +332,7 @@ hack_all_symbols (hack_symbol *func, const char *data)
       for (sym = symtab[h]; sym != NULL; sym = next)
 	{
 	  next = SYMBOL_NEXT (sym);
-	  (*func) (sym, data);
+	  func (sym, data);
 	}
     }
 }
@@ -358,7 +350,7 @@ symtab_debug (void)
   int delete;
   static int i;
 
-  while (next_token (&td) == TOKEN_WORD)
+  while (next_token (&td, NULL, NULL, false, "<debug>") == TOKEN_WORD)
     {
       text = TOKEN_DATA_TEXT (&td);
       if (*text == '_')
@@ -372,7 +364,7 @@ symtab_debug (void)
       s = lookup_symbol (text, SYMBOL_LOOKUP);
 
       if (s == NULL)
-	printf ("Name `%s' is unknown\n", text);
+	xprintf ("Name `%s' is unknown\n", text);
 
       if (delete)
 	(void) lookup_symbol (text, SYMBOL_DELETE);
@@ -388,17 +380,17 @@ symtab_print_list (int i)
   symbol *sym;
   size_t h;
 
-  printf ("Symbol dump #%d:\n", i);
+  xprintf ("Symbol dump #%d:\n", i);
   for (h = 0; h < hash_table_size; h++)
     for (sym = symtab[h]; sym != NULL; sym = sym->next)
-      printf ("\tname %s, bucket %lu, addr %p, next %p, "
-	      "flags%s%s%s, pending %d\n",
-	      SYMBOL_NAME (sym),
-	      (unsigned long int) h, sym, SYMBOL_NEXT (sym),
-	      SYMBOL_TRACED (sym) ? " traced" : "",
-	      SYMBOL_SHADOWED (sym) ? " shadowed" : "",
-	      SYMBOL_DELETED (sym) ? " deleted" : "",
-	      SYMBOL_PENDING_EXPANSIONS (sym));
+      xprintf ("\tname %s, bucket %lu, addr %p, next %p, "
+	       "flags%s%s%s, pending %d\n",
+	       SYMBOL_NAME (sym),
+	       (unsigned long int) h, sym, SYMBOL_NEXT (sym),
+	       SYMBOL_TRACED (sym) ? " traced" : "",
+	       SYMBOL_SHADOWED (sym) ? " shadowed" : "",
+	       SYMBOL_DELETED (sym) ? " deleted" : "",
+	       SYMBOL_PENDING_EXPANSIONS (sym));
 }
 
 #endif /* DEBUG_SYM */
